@@ -8,12 +8,11 @@ To report an issue, please use: {}
 
 import os
 import json
-import logging
 import shlex
 import subprocess  # nosec B404:blacklist import_subprocess
 from functools import wraps
 from getpass import getpass
-from typing import List, Dict, Any, Optional
+from typing import List, Dict, Any, Optional, Callable
 
 from .progress import CliProgress
 from .. import VERSION_WITH_DEPS
@@ -29,10 +28,15 @@ from .cli_builder import (
     decorate,
 )
 from ..utils.log import (
-    exception_to_message,
-    add_stream_handler_to_logger,
-    add_rotating_file_handler_to_logger,
+    log_to_stream,
+    log_to_rotating_file,
     create_logger,
+    log_runtime_info,
+)
+from ..utils.error_handling import (
+    exit_on_exception,
+    log_exceptions,
+    error_report_hint_at_exit,
 )
 from ..workflows.config import create as create_config
 from ..workflows.transfer import transfer as workflows_transfer
@@ -40,7 +44,7 @@ from ..workflows.decrypt import decrypt as workflows_decrypt
 from ..workflows.encrypt import encrypt as workflows_encrypt
 from ..workflows.request_sigs import request_sigs
 from ..workflows.upload_keys import upload_keys
-from ..protocols import parse_protocol, __all__ as available_protocols
+from ..protocols import Protocol, parse_protocol, __all__ as available_protocols, sftp
 from ..core.versioncheck import check_version
 from ..core.error import UserError
 from .. import URL_READTHEDOCS, URL_GITLAB_ISSUES
@@ -85,14 +89,12 @@ def get_passphrase_from_cmd(passphrase_cmd: str) -> str:
         ) from e
 
 
-@exception_to_message(UserError, logger)
 def get_passphrase_from_cmd_or_prompt(msg: str, passphrase_cmd: Optional[str]) -> str:
     if passphrase_cmd:
         return get_passphrase_from_cmd(passphrase_cmd)
     return getpass(msg)
 
 
-@exception_to_message(UserError, logger)
 @wraps(workflows_encrypt)
 def encrypt(
     *args,
@@ -138,7 +140,6 @@ def decrypt(*args, dry_run: bool, passphrase_cmd: Optional[str], **kwargs):
     return workflows_decrypt(*args, dry_run=dry_run, **kwargs)
 
 
-@exception_to_message((UserError, FileNotFoundError), logger)
 def transfer(
     files: List[str],
     *,
@@ -168,18 +169,29 @@ def transfer(
             "or 'connection' "
             "has to be given as an argument"
         )
-    for pw_arg in protocol.required_password_args(**protocol_args):
+    specific_protocol_args = {
+        sftp.Protocol: {"two_factor_callback": two_factor_callback}
+    }
+    try:
+        protocol_instance: Protocol = protocol(
+            **protocol_args, **specific_protocol_args.get(protocol, {})
+        )
+    except TypeError as e:
+        raise UserError(format(e).replace("__init__", protocol.__module__)) from None
+    for pw_arg in protocol_instance.required_password_args():
         if protocol_args.get(pw_arg) is None:
-            protocol_args[pw_arg] = get_passphrase_from_cmd_or_prompt(
-                f"Please enter the password for the argument `{pw_arg}`:",
-                passphrase_cmd,
+            setattr(
+                protocol_instance,
+                pw_arg,
+                get_passphrase_from_cmd_or_prompt(
+                    f"Please enter the password for the argument `{pw_arg}`:",
+                    passphrase_cmd,
+                ),
             )
     return workflows_transfer(
         files,
-        protocol=protocol.upload,
-        protocol_args=protocol_args,
+        protocol=protocol_instance,
         config=config,
-        two_factor_callback=two_factor_callback,
         dry_run=dry_run,
         verify_dtr=verify_dtr,
         progress=progress,
@@ -187,9 +199,9 @@ def transfer(
 
 
 def load_config_check():
-    cfg = exception_to_message(logger=logger)(load_config)()
+    cfg = load_config()
     if not cfg.offline and cfg.check_version:
-        exception_to_message(logger=logger)(check_version)(cfg.repo_url)
+        check_version(cfg.repo_url)
     return cfg
 
 
@@ -211,11 +223,6 @@ class Cli(Subcommands):
     verify_dtr_override = dict(
         help="Verify DTR ID against online database.", default=None
     )
-    logger = logging.getLogger()
-    add_rotating_file_handler_to_logger(
-        logger, log_dir=config.log_dir, file_max_number=config.log_max_file_number
-    )
-    add_stream_handler_to_logger(logger)
     subcommands = (
         Subcommand(
             decorate(
@@ -325,7 +332,25 @@ class Cli(Subcommands):
     )
 
 
+def when(
+    condition: bool, decorator: Callable[[Callable], Callable]
+) -> Callable[[Callable], Callable]:
+    if condition:
+        return decorator
+    return lambda x: x
+
+
+log_to_rotating_file(
+    log_dir=Cli.config.log_dir, file_max_number=Cli.config.log_max_file_number
+)
+log_to_stream()
+
+
+@exit_on_exception
+@when(Cli.config.error_reports, error_report_hint_at_exit(Cli.config))
+@log_exceptions(logger)
 def run():
+    log_runtime_info(logger)
     if Cli():
         return 0
     return 1
