@@ -38,7 +38,7 @@ def check_version(repo_url: str, gui_formatting: bool = False):
 def get_latest_version(repo_url: str):
     try:
         url = repo_url + "/simple/" + __project_name__ + "/"
-        with urlopen(url) as response:  # nosec
+        with urlopen(url) as response:
             versions = re.findall(
                 __project_name__.encode() + b"-([0-9]*.[0-9]*.[0-9]*).tar.gz",
                 response.read(),
