@@ -2,12 +2,54 @@ from functools import partial
 from pathlib import Path
 from typing import Callable, Optional, Tuple
 
-from .pyside import QtCore, QtWidgets
-
 from .model import AppData
 from .parallel import Worker
+from .pyside import QtCore, QtWidgets, QAction, QtGui
 from ..utils.progress import ProgressInterface
 
+EXTRA_HEIGHT = 1
+MACOS_TOOLTIP_BG_COLOR = "#ffffca"
+
+
+def is_macos():
+    return QtCore.QSysInfo.productType() in ("osx", "macos")
+
+
+class Action(QAction):
+    def __init__(
+        self,
+        icon: QtGui.QIcon,
+        text: str,
+        parent: Optional[QtWidgets.QWidget] = None,
+    ):
+        super().__init__(icon, text, parent)
+        if is_macos():
+            self.setToolTip(
+                f"<p style='background-color: {MACOS_TOOLTIP_BG_COLOR};'>{text}</p>"
+            )
+
+
+class SpinBox(QtWidgets.QSpinBox):
+    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
+        super().__init__(parent)
+        if is_macos():
+            self.setMinimumHeight(self.sizeHint().height() + EXTRA_HEIGHT)
+
+
+class LineEdit(QtWidgets.QLineEdit):
+    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
+        super().__init__(parent)
+        if is_macos():
+            self.setMinimumHeight(self.sizeHint().height() + EXTRA_HEIGHT)
+
+
+class ToolBar(QtWidgets.QToolBar):
+    def __init__(self, title: str, parent: Optional[QtWidgets.QWidget] = None):
+        super().__init__(title, parent)
+        if is_macos():
+            self.setStyleSheet("background: transparent;border-width: 0px;")
+        self.setIconSize(QtCore.QSize(20, 20))
+
 
 class MandatoryLabel(QtWidgets.QLabel):
     """A label extension which appends a '*' to label's end to mark the field as required."""
@@ -35,6 +77,25 @@ class SelectionButton(QtWidgets.QPushButton):
         self.setEnabled(bool(len(self._selection_model.selectedRows())))
 
 
+class SelectionAction(Action):
+    """An action extension which connects this action to given selection model.
+
+    The action is disabled by default. And gets enabled when the selection has, at least,
+    one selected row.
+    """
+
+    def __init__(self, *args, selection_model: QtCore.QItemSelectionModel):
+        super().__init__(*args)
+        self.setEnabled(False)
+        self._selection_model = selection_model
+        selection_model.selectionChanged.connect(self.selection_changed)
+
+    def selection_changed(self):
+        # Following works better than using 'QItemSelection', especially in cases
+        # where multiple selection is possible
+        self.setEnabled(bool(len(self._selection_model.selectedRows())))
+
+
 class GuiProgress(QtCore.QObject, ProgressInterface):
     updated = QtCore.Signal(int)
 
@@ -55,12 +116,14 @@ class ConsoleWidget(QtWidgets.QGroupBox):
         super().__init__(title, parent)
         self.textbox = QtWidgets.QPlainTextEdit()
         self.textbox.setReadOnly(True)
-        btn_clear_console = QtWidgets.QPushButton("Clear console")
+        btn_clear_console = QtWidgets.QPushButton(
+            QtGui.QIcon(":icon/feather/slash.png"), ""
+        )
+        btn_clear_console.setToolTip("Clear console")
         btn_clear_console.clicked.connect(self.clear)
-        layout = QtWidgets.QVBoxLayout()
+        layout = QtWidgets.QHBoxLayout(self)
         layout.addWidget(self.textbox)
-        layout.addWidget(btn_clear_console)
-        self.setLayout(layout)
+        layout.addWidget(btn_clear_console, alignment=QtCore.Qt.AlignTop)
 
     def clear(self) -> None:
         self.textbox.clear()
@@ -77,12 +140,19 @@ class PathInput:
 
     def __init__(self, directory=True, path: Optional[Path] = Path.home(), parent=None):
         self.parent = parent
-        self.text = QtWidgets.QLineEdit(parent)
+        self.text = LineEdit(parent)
         self.text.setReadOnly(True)
-        self.btn = QtWidgets.QPushButton("Change location")
+        self.btn = QtWidgets.QPushButton(
+            QtGui.QIcon(f":icon/feather/{'folder' if directory else 'file'}.png"),
+            "",
+        )
+        self.btn.setToolTip("Change location")
         self.btn.clicked.connect(partial(self._update_location, directory))
-        # Additional button to clear the selected path
-        self.btn_clear = QtWidgets.QPushButton("Clear")
+        # Additional action to clear the selected path
+        self.btn_clear = QtWidgets.QPushButton(
+            QtGui.QIcon(":icon/feather/slash.png"), ""
+        )
+        self.btn_clear.setToolTip("Clear location")
         self.btn_clear.clicked.connect(self._clear_location)
         self.update_path(path)
 
@@ -110,15 +180,6 @@ class PathInput:
     def _clear_location(self):
         self.update_path(None)
 
-    @property
-    def btn_label(self) -> str:
-        """Button label"""
-        return self.btn.text()
-
-    @btn_label.setter
-    def btn_label(self, label: str):
-        self.btn.setText(label)
-
     @property
     def status_tip(self) -> str:
         return self.text.statusTip()
@@ -151,7 +212,7 @@ class TabMixin:
 
     def create_run_panel(self, panel_name: str, action: Callable, action_name: str):
         self.run_panel = QtWidgets.QGroupBox(panel_name)
-        layout = QtWidgets.QHBoxLayout()
+        layout = QtWidgets.QHBoxLayout(self.run_panel)
         self.btn_test = TabMixin._create_disabled_button("Test")
         # On pressed button, make sure that the focus switches to that button (Mac specific issue)
         self.btn_test.pressed.connect(self.btn_test.setFocus)
@@ -162,7 +223,6 @@ class TabMixin:
         self.btn_run.clicked.connect(action)
         layout.addWidget(self.btn_test)
         layout.addWidget(self.btn_run)
-        self.run_panel.setLayout(layout)
 
     def add_worker_actions(self, worker: Worker):
         """Attach GUI-updating signals to worker"""
@@ -183,12 +243,31 @@ def run_dialog(parent, msg: str, password: bool = True):
     return dialog_pwd.textValue()
 
 
-def show_warning(title: str, text: str, parent: Optional[QtWidgets.QWidget] = None):
+def warning_callback(
+    title: str, parent: Optional[QtWidgets.QWidget] = None
+) -> Callable:
+    msg_warn = warning_dialog(title, parent=parent)
+
+    def _show_warning(msg: str):
+        msg_warn.setText(format(msg))
+        msg_warn.exec_()
+
+    return _show_warning
+
+
+def warning_dialog(
+    title: str, text: Optional[str] = None, parent: Optional[QtWidgets.QWidget] = None
+) -> QtWidgets.QMessageBox:
     msg = QtWidgets.QMessageBox(parent=parent)
     msg.setWindowTitle(title)
     msg.setIcon(QtWidgets.QMessageBox.Warning)
-    msg.setText(text)
-    msg.show()
+    if text is not None:
+        msg.setText(text)
+    return msg
+
+
+def show_warning(title: str, text: str, parent: Optional[QtWidgets.QWidget] = None):
+    warning_dialog(title, text, parent).show()
 
 
 def create_verify_dtr_checkbox(
