@@ -415,7 +415,7 @@ class KeyDownloadDialog(QtWidgets.QDialog):
             crypt.search_keyserver,
             self.search_string.text(),
             self.parent().app_data.config.keyserver_url,
-            report_config=self.app_data.config,
+            report_config=self.parent().app_data.config,
         )
         worker.signals.result.connect(
             lambda keys: self.key_list_view.model().setStringList(
@@ -436,7 +436,7 @@ class KeyDownloadDialog(QtWidgets.QDialog):
                 key_ids,
                 self.parent().app_data.config.keyserver_url,
                 self.parent().app_data.config.gpg_store,
-                report_config=self.app_data.config,
+                report_config=self.parent().app_data.config,
             )
             worker.signals.result.connect(lambda x: self.parent().update_public_keys())
             worker.signals.finished.connect(lambda: self.btn_download.setEnabled(True))
@@ -558,7 +558,7 @@ class KeyGenDialog(QtWidgets.QDialog):
             self.text_pass_repeat.text(),
             self.parent().app_data.config.gpg_store,
             forward_errors=show_warning,
-            report_config=self.app_data.config,
+            report_config=self.parent().app_data.config,
         )
         worker.signals.result.connect(self.post_key_creation)
         worker.signals.finished.connect(lambda: self.btn_run.setEnabled(True))
