@@ -5,9 +5,8 @@ from functools import partial
 from pathlib import Path
 from typing import List, Optional, Callable, Tuple, Union
 
-from . import workflow
 from ..utils.config import Config
-from ..utils.log import create_logger
+from ..utils.log import create_logger, log_timing
 from ..core.archive import (
     write_tar,
     ArchiveInMemoryFile,
@@ -33,6 +32,7 @@ from ..core.crypt import (
 from ..core.checksum import write_checksums, compute_checksum_on_write
 from ..core.error import UserError
 from ..core.metadata import MetaData, alnum_str, Purpose, HexStr1024, HexStr256
+from ..core.secret import Secret
 from ..utils.progress import ProgressInterface, subprogress, progress_file_iter
 
 DATE_FMT_FILENAME = "%Y%m%dT%H%M%S"
@@ -107,7 +107,7 @@ check_compression_level = check_integer_in_range(min_value=0, max_value=9)
 
 
 # pylint: disable=too-many-statements
-@workflow(logger, (UserError, FileNotFoundError, PermissionError))
+@log_timing(logger)
 def encrypt(
     files: List[str],
     *,
@@ -115,7 +115,7 @@ def encrypt(
     recipient: List[str],
     dtr_id: Optional[int] = None,
     sender: Optional[str] = None,
-    passphrase: Optional[str] = None,
+    passphrase: Optional[Secret] = None,
     output_name: Optional[str] = None,
     dry_run: bool = False,
     offline: bool = False,
@@ -347,7 +347,7 @@ def encrypt(
     return output_name
 
 
-def enforce_passphrase(passphrase: Optional[str]) -> str:
+def enforce_passphrase(passphrase: Optional[Secret]) -> Secret:
     if passphrase is None:
         raise ValueError("No password given")
     return passphrase
