@@ -1,6 +1,5 @@
-from typing import List
+from typing import Sequence
 
-from . import workflow
 from ..core.crypt import (
     upload_keys as crypt_upload_keys,
     verify_key_length,
@@ -14,8 +13,7 @@ from ..utils.config import Config
 logger = create_logger(__name__)
 
 
-@workflow(logger, UserError)
-def upload_keys(key_ids: List[str], *, config: Config):
+def upload_keys(key_ids: Sequence[str], *, config: Config):
     """Upload keys"""
     if config.offline:
         raise UserError("Uploading keys is not possible in the offline mode.")
